﻿using System;
using System.Reflection;
using System.Resources;
using System.Runtime.InteropServices;

namespace TestProject
{
    /// <summary>
    /// 어셈블리 헬퍼
    /// </summary>
    public class AssemblyHelper
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 제목
        /// </summary>
        public string Title = string.Empty;
        
        /// <summary>
        /// 설명
        /// </summary>
        public string Description = string.Empty;
        
        /// <summary>
        /// 회사
        /// </summary>
        public string Company = string.Empty;
        
        /// <summary>
        /// 제품
        /// </summary>
        public string Product = string.Empty;
        
        /// <summary>
        /// 저작권
        /// </summary>
        public string Copyright = string.Empty;
        
        /// <summary>
        /// 상표
        /// </summary>
        public string Trademark = string.Empty;
        
        /// <summary>
        /// 어셈블리 버전
        /// </summary>
        public string AssemblyVersion = string.Empty;
        
        /// <summary>
        /// 파일 버전
        /// </summary>
        public string FileVersion = string.Empty;
        
        /// <summary>
        /// GUID
        /// </summary>
        public string GUID = string.Empty;

        /// <summary>
        /// 중립 언어
        /// </summary>
        public string NeutralLanguage = string.Empty;

        /// <summary>
        /// COM 표시 여부
        /// </summary>
        public bool IsCOMVisible = false;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - AssemblyHelper(assembly)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="assembly">어셈블리</param>
        public AssemblyHelper(Assembly assembly)
        {
            AssemblyTitleAttribute assemblyTitleAttribute = GetAssemblyAttribute<AssemblyTitleAttribute>(assembly);

            if(assemblyTitleAttribute != null)
            {
                Title = assemblyTitleAttribute.Title;
            }

            AssemblyDescriptionAttribute assemblyDescriptionAttribute = GetAssemblyAttribute<AssemblyDescriptionAttribute>(assembly);

            if(assemblyDescriptionAttribute != null)
            {
                Description = assemblyDescriptionAttribute.Description;
            }

            AssemblyCompanyAttribute assemblyCompanyAttribute = GetAssemblyAttribute<AssemblyCompanyAttribute>(assembly);

            if(assemblyCompanyAttribute != null)
            {
                Company = assemblyCompanyAttribute.Company;
            }

            AssemblyProductAttribute assemblyProductAttribute = GetAssemblyAttribute<AssemblyProductAttribute>(assembly);

            if(assemblyProductAttribute != null)
            {
                Product = assemblyProductAttribute.Product;
            }

            AssemblyCopyrightAttribute assemblyCopyrightAttribute = GetAssemblyAttribute<AssemblyCopyrightAttribute>(assembly);

            if(assemblyCopyrightAttribute != null)
            {
                Copyright = assemblyCopyrightAttribute.Copyright;
            }

            AssemblyTrademarkAttribute assemblyTrademarkAttribute = GetAssemblyAttribute<AssemblyTrademarkAttribute>(assembly);

            if(assemblyTrademarkAttribute != null)
            {
                Trademark = assemblyTrademarkAttribute.Trademark;
            }

            AssemblyVersion = assembly.GetName().Version.ToString();

            AssemblyFileVersionAttribute assemblyFileVersionAttribute = GetAssemblyAttribute<AssemblyFileVersionAttribute>(assembly);

            if(assemblyFileVersionAttribute != null)
            {
                FileVersion = assemblyFileVersionAttribute.Version;
            }

            GuidAttribute guidAttribute = GetAssemblyAttribute<GuidAttribute>(assembly);

            if(guidAttribute != null)
            {
                GUID = guidAttribute.Value;
            }

            NeutralResourcesLanguageAttribute neutralResourcesLanguageAttribute = GetAssemblyAttribute<NeutralResourcesLanguageAttribute>(assembly);

            if(neutralResourcesLanguageAttribute != null)
            {
                NeutralLanguage = neutralResourcesLanguageAttribute.CultureName;
            }

            ComVisibleAttribute comVisibleAttribute = GetAssemblyAttribute<ComVisibleAttribute>(assembly);

            if(comVisibleAttribute != null)
            {
                IsCOMVisible = comVisibleAttribute.Value;
            }
        }

        #endregion
        #region 생성자 - AssemblyHelper()

        /// <summary>
        /// 생성자
        /// </summary>
        public AssemblyHelper() : this(Assembly.GetExecutingAssembly())
        {
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 어셈블리 어트리뷰트 구하기 - GetAssemblyAttribute<T>(assembly)

        /// <summary>
        /// 어셈블리 어트리뷰트 구하기
        /// </summary>
        /// <typeparam name="T">어트리뷰트 타입</typeparam>
        /// <param name="assembly">어셈블리</param>
        /// <returns>어셈블리 어트리뷰트</returns>
        public static T GetAssemblyAttribute<T>(Assembly assembly) where T : Attribute
        {
            object[] attributeArray = assembly.GetCustomAttributes(typeof(T), true);

            if((attributeArray == null) || (attributeArray.Length == 0))
            {
                return null;
            }

            return (T)attributeArray[0];
        }

        #endregion
    }
}